function [ RESAMP, XPIX, YPIX ] = imresamp(IMAGE, LY, LX, TYPE )
% [ RESAMP, XPIX, YPIX ] = imresamp(IMAGE, LY, LX, TYPE )
%
% Resample grayscale IMAGE (size MxN), return the result as RESAMP (size RMxRN).
% LY is the resampling factor in the column direction,
% LX is the resampling factor in the row direction.
% XPIX (1xRN) gives the input X value corresponding to each output X.
% YPIX (1xRM) gives the input Y value corresponding to each output Y.
%
% If [M,N] = size(IMAGE), then the size of RESAMP will be
% size(RESAMP) = [ 1+floor(LY*(M-1)), 1+floor(LX*(N-1)) ];
%
% The (1,1) point of RESAMP will be at the same point as the (1,1) point of IMAGE.
% Other coordinates are stretched by LX and LY.
%
% Filtering is performed using a 20-point approximation to a sinc function.
% Pixels beyond the edge are assumed to have the same value as the nearest edge pixel.
%
% If TYPE is a string, other types of interpolation may be selected.
% Available choices are 'spline', 'sinc', 'linear', and 'gaussian'.
%

if nargin<4, TYPE='sinc'; end
[M,N] = size(IMAGE);

% Create axis labels mapping the new coordinate space back into the old one
XPIX = 1:(1/LX):N;
YPIX = 1:(1/LY):M;

% Find the size of the resampled image
RN = length(XPIX);
RM = length(YPIX);

% Space for storing X-interpolated and fully interpolated images
xinterp = zeros(M, RN);
RESAMP = zeros(RM, RN);

% Spline interpolation: create the initial axis set, then spline interpolate
if TYPE(1:2) == 'sp',
  for iy=1:M,
    xinterp(iy,:) = spline(1:N, IMAGE(iy, :), XPIX);
  end;
  for ix=1:RN,
    RESAMP(:,ix) = spline(1:N, xinterp(:, ix)', YPIX)';
  end;
  return;
end

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Separable filtering.
% This is rather hairy code, but I think it's faster this way.
% The idea is that, since the filter is usually smaller than the image,
% we step through the filter points rather than stepping through the image points.
% 
% xf [1xRN] gives the nearest INPUT column to the left of each OUTPUT column.
% d [1xND] gives the index of each filter tap, typically from -something up to +something.
% xh [NDxRN] gives ND filter coefficients for each of the RN output columns.
%
% To create output column OUTCOL(ix), we accumulate, for each possible d(id),
% the product of the input column INCOL(xf(ix)+d(id)) times the coefficient xh(id,ix).
% 

xf = floor(XPIX);
yf = floor(YPIX);
  
if TYPE(1)=='l',
  % Linear nearest-neighbor interpolation
  xd = [0 1];
  yd = [0 1];
  xh = [ 1-abs(XPIX-xf); 1-abs(XPIX-xf-1) ];
  yh = [ 1-abs(YPIX-yf); 1-abs(YPIX-yf-1) ];
else,
  % Interpolation filter is in "minimal units" -- that is,
  % input units if we're upsampling (LX>1),
  % or in output units if we're downsampling (LX<1).
  MinX = min(LX,1);
  MinY = min(LY,1);
  
  %   sinc and Gaussian filters both use 20 taps --
  % that's 20 taps in input units if we're upsampling (LX>1),
  % 20 taps in output units if we're downsampling (LX<1)
  xd = -round(9/MinX):round(10/MinX);
  yd = -round(9/MinY):round(10/MinY);
  xh = zeros(length(xd),RN);
  yh = zeros(length(yd),RM);

  % xp gives the displacement from input point xf(ix)+d(id) to output point ix, in minimal units.
  xp = MinX * ( repmat(XPIX-xf, [length(xd) 1]) - repmat(xd', [1 RN]) );
  xp(xp==0) = repmat(1e-10, size(find(xp==0)));
  yp = MinY * ( repmat(YPIX-yf, [length(yd) 1]) - repmat(yd', [1 RM]) );
  yp(yp==0) = repmat(1e-10, size(find(yp==0)));
  
  % Calculate filter coefficients
  if TYPE(1) == 'g',
    % Filter coefficients are from a Gaussian kernel with SD of 0.5
    xh = exp(-xp.^2/(2*.25))/sqrt(pi*.25);
    yh = exp(-yp.^2/(2*.25))/sqrt(pi*.25);
  else,
    % Default: filter coefficients are from a sinc function
    xh = sin(pi*xp) ./ (pi*xp);
    yh = sin(pi*yp) ./ (pi*yp);
  end
end;

% Normalize to unity in each individual dimension, but of course the whole 2D filter may not be
% normalized!
xh = xh ./ repmat(sum(xh),[length(xd) 1]);
yh = yh ./ repmat(sum(yh),[length(yd) 1]);

% Separable interpolation.  Pixels beyond the image boundary are filled in using edge values
for id=1:length(xd),
    xinterp = xinterp + repmat(xh(id,:),[M 1]) .* IMAGE( :, max(1,min(N,xf+xd(id)))); 
end;
for id=1:length(yd),
    RESAMP = RESAMP + repmat(yh(id,:)', [1 RN]) .* xinterp( max(1,min(M,yf+yd(id))), : );
end;
