function [Y, ROI]=otl2roi( X, H )
% [Y,ROI]=otl2roi(X, H)
%
% Use otl2map and heuristics to convert X (otl array) to A (ROI image)
%
% X [3xNPTS] = top 2 rows are X and Y coordinates of all OTL points,
%              bottom row is the color code for each region
%
% Y [3xM] = an otl array like X, but containing interior points too
% A [512x512] = an ROI map, upsampled by factor of two from the OTL in X.
%
% Heuristic Method:
%
% 1) Primary color is the color with fewer multilines, but at least one
% 2) Create MAP from the primary color.
% 3) For all ML's in secondary color: try to close the ML by
%  pasting in segments of the largest ML in the primary color.
% 4) Copy magenta and yellow lines or multilines as open regions.
%
% Mark Hasegawa-Johnson
% 3/18/2002

% Initialize ROI and Y
ROI=zeros([512 512]);
Y=zeros(3,0);

% Make adjustments to H as necessary
if nargin>1, if isstruct(H), if isfield(H,'npixels'), if size(H.npixels)==2,
            H.npixels = [ H.npixels 128 ];
         end, end, end, end

% COLORS matrix --- first two elements must be 'cg'
COLORS = 'cgrmybwk';

% If X is null, Y is null
if isempty(X), 
   return;
elseif size(X,1)==2,
   % If X has only 2 rows, assume all points are green
   X = [ X; repmat(double('g'),[1 size(X,2)]) ];
end

% Get the multiline objects
MLINE = {{[]} {[]}};
for ci=1:2,
   % Get upsampled coordinates
   O{ci} = 2*X(1:2,X(3,:)==COLORS(ci));
   
   if ~isempty(O{ci}),
      % Get multiline objects for C and G
      [ foo, MLINE{ci} ] = otl2ml(O{ci}(1,:),O{ci}(2,:),[512 512],4);
   end
end

   
% Try to complete all segments of either cyan or green
% using segments from the FIRST ML of the opposite color
for ci=1:2,
   P = MLINE{3-ci}{1};
   for i=1:length(MLINE{ci}),
      S = MLINE{ci}{i};
      
      if ~isempty(S),
         if ~isempty(P),
            % First, find the points on P that are closest to endpoints of S
            [ firstmindist, fmdi ] = min(sum((P-repmat(S(1,:),[size(P,1) 1])).^2,2));
            [ lastmindist, lmdi ] = min(sum((P-repmat(S(size(S,1),:),[size(P,1) 1])).^2,2));
   
            % If the distance is less than 20 pixels in each case, close the loop
            if lastmindist < 20 & firstmindist < 20,
               if lmdi>fmdi,
                  S = [ S; P(lmdi:-1:fmdi,:) ];
               else
                  S = [ S; P(lmdi:fmdi,:) ];
               end
            end
         end
         
         
         % Now create a map using these points
         MAP = otl2map(S(:,2),S(:,1),[512 512],20);
   
         % Find the interior and exterior points
         INTPTS = find(MAP==2);
   
         % Color interior points on the primary ROI image
         ROI(INTPTS) = COLORS(ci);
      end
   end
end


% Check all of the other colors, and put points into ROI
for C=COLORS(3:8),
   I=find(otl2ml(2*X(1,X(3,:)==C),2*X(2,X(3,:)==C),[512 512],4));
   if ~isempty(I),
      ROI(I)=C;
   end
end

% Extract column and row indices for the OTL array Y
Y = zeros(3,0);
for i=1:length(COLORS),
   [I,J] = find(ROI==COLORS(i));
   Y = [ Y, [J'/2; I'/2; repmat(double(COLORS(i)),[1 length(I)])] ];
end

